# Copyright (C) 2020 Analog Devices, Inc.
#
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification,
# are permitted provided that the following conditions are met:
#     - Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     - Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in
#       the documentation and/or other materials provided with the
#       distribution.
#     - Neither the name of Analog Devices, Inc. nor the names of its
#       contributors may be used to endorse or promote products derived
#       from this software without specific prior written permission.
#     - The use of this software may or may not infringe the patent rights
#       of one or more patent holders.  This license does not release you
#       from the requirement that you obtain separate licenses from these
#       patent holders to use this software.
#     - Use of the software either in source or binary form, must be run
#       on or directly connected to an Analog Devices Inc. component.
#
# THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
# INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE ARE DISCLAIMED.
#
# IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, INTELLECTUAL PROPERTY
# RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
# STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
# THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from typing import Dict, List

from adi.context_manager import context_manager
from adi.rx_tx import rx_tx


def _map_to_dict(paths, ch):
    fddc, cddc, adc = ch.attrs["label"].value.split("->")
    if adc not in paths.keys():
        paths[adc] = {}
    if cddc not in paths[adc].keys():
        paths[adc][cddc] = {}
    if fddc not in paths[adc][cddc].keys():
        paths[adc][cddc][fddc] = {"channels": [ch._id]}
    else:
        paths[adc][cddc][fddc]["channels"].append(ch._id)
    return paths


def _sortconv(chans_names, noq=False, dds=False):
    tmpI = filter(lambda k: "_i" in k, chans_names)
    tmpQ = filter(lambda k: "_q" in k, chans_names)

    def ignoreadc(w):
        return int(w[len("voltage") : w.find("_")])

    def ignorealt(w):
        return int(w[len("altvoltage") :])

    chans_names_out = []
    if dds:
        filt = ignorealt
        tmpI = chans_names
        noq = True
    else:
        filt = ignoreadc

    tmpI = sorted(tmpI, key=filt)
    tmpQ = sorted(tmpQ, key=filt)
    for i in range(len(tmpI)):
        chans_names_out.append(tmpI[i])
        if not noq:
            chans_names_out.append(tmpQ[i])

    return chans_names_out


class ad9081(rx_tx, context_manager):
    """ AD9081 Mixed-Signal Front End (MxFE) """

    _complex_data = True
    _rx_channel_names: List[str] = []
    _tx_channel_names: List[str] = []
    _tx_control_channel_names: List[str] = []
    _rx_coarse_ddc_channel_names: List[str] = []
    _tx_coarse_duc_channel_names: List[str] = []
    _rx_fine_ddc_channel_names: List[str] = []
    _tx_fine_duc_channel_names: List[str] = []
    _dds_channel_names: List[str] = []
    _device_name = ""

    _rx_attr_only_channel_names: List[str] = []
    _tx_attr_only_channel_names: List[str] = []

    _path_map: Dict[str, Dict[str, Dict[str, List[str]]]] = {}

    def __init__(self, uri=""):

        context_manager.__init__(self, uri, self._device_name)
        # Default device for attribute writes
        self._ctrl = self._ctx.find_device("axi-ad9081-rx-hpc")
        # Devices with buffers
        self._rxadc = self._ctx.find_device("axi-ad9081-rx-hpc")
        self._txdac = self._ctx.find_device("axi-ad9081-tx-hpc")

        # Get DDC and DUC mappings
        paths = {}

        for ch in self._rxadc.channels:
            if "label" in ch.attrs:
                paths = _map_to_dict(paths, ch)
        self._path_map = paths

        # Get data + DDS channels
        for ch in self._rxadc.channels:
            if ch.scan_element and not ch.output:
                self._rx_channel_names.append(ch._id)
        for ch in self._txdac.channels:
            if ch.scan_element:
                self._tx_channel_names.append(ch._id)
            else:
                self._dds_channel_names.append(ch._id)

        # Sort channel names
        self._rx_channel_names = _sortconv(self._rx_channel_names)
        self._tx_channel_names = _sortconv(self._tx_channel_names)
        self._dds_channel_names = _sortconv(self._dds_channel_names, dds=True)

        # Map unique attributes to channel properties
        self._rx_fine_ddc_channel_names = []
        self._rx_coarse_ddc_channel_names = []
        self._tx_fine_duc_channel_names = []
        self._tx_coarse_duc_channel_names = []
        for converter in paths:
            for cdc in paths[converter]:
                channels = []
                for fdc in paths[converter][cdc]:
                    channels += paths[converter][cdc][fdc]["channels"]
                channels = [name for name in channels if "_i" in name]
                if "ADC" in converter:
                    self._rx_coarse_ddc_channel_names.append(channels[0])
                    self._rx_fine_ddc_channel_names += channels
                else:
                    self._tx_coarse_duc_channel_names.append(channels[0])
                    self._tx_fine_duc_channel_names += channels

        rx_tx.__init__(self)
        self.rx_buffer_size = 2 ** 16

    def _get_iio_attr_str_single(self, channel_name, attr, output):
        # This is overridden by subclasses
        return self._get_iio_attr_str(channel_name, attr, output)

    def _set_iio_attr_str_single(self, channel_name, attr, output, value):
        # This is overridden by subclasses
        return self._set_iio_attr(channel_name, attr, output, value)

    def _get_iio_attr_single(self, channel_name, attr, output):
        # This is overridden by subclasses
        return self._get_iio_attr(channel_name, attr, output)

    def _set_iio_attr_single(self, channel_name, attr, output, value):
        # This is overridden by subclasses
        return self._set_iio_attr(channel_name, attr, output, value)

    def _get_iio_dev_attr_single(self, attr):
        # This is overridden by subclasses
        return self._get_iio_dev_attr(attr)

    def _set_iio_dev_attr_single(self, attr, value):
        # This is overridden by subclasses
        return self._set_iio_dev_attr(attr, value)

    @property
    def path_map(self):
        """ path_map: Map of channelizers both coarse and fine to
            individual driver channel names
        """
        return self._path_map

    def write_pfilt_config(self, value):
        """ Load a new PFILT configuration file
            Input is path to PFILT configuration file. Please see
            driver documentation about PFILT generation and limitations
        """
        with open(value, "r") as file:
            data = file.read()
        self._set_iio_dev_attr_str("filter_fir_config", data)

    # we cannot really get the profile. The driver will just throw EPERM
    pfilt_config = property(None, write_pfilt_config)

    @property
    def rx_channel_nco_frequencies(self):
        """rx_channel_nco_frequencies: Receive path fine DDC NCO frequencies
        """
        return self._get_iio_attr_vec(
            self._rx_fine_ddc_channel_names, "channel_nco_frequency", False
        )

    @rx_channel_nco_frequencies.setter
    def rx_channel_nco_frequencies(self, value):
        self._set_iio_attr_int_vec(
            self._rx_fine_ddc_channel_names, "channel_nco_frequency", False, value
        )

    @property
    def rx_channel_nco_phases(self):
        """rx_channel_nco_phases: Receive path fine DDC NCO phases
        """
        return self._get_iio_attr_vec(
            self._rx_fine_ddc_channel_names, "channel_nco_phase", False
        )

    @rx_channel_nco_phases.setter
    def rx_channel_nco_phases(self, value):
        self._set_iio_attr_int_vec(
            self._rx_fine_ddc_channel_names, "channel_nco_phase", False, value,
        )

    @property
    def rx_main_nco_frequencies(self):
        """rx_main_nco_frequencies: Receive path coarse DDC NCO frequencies
        """
        return self._get_iio_attr_vec(
            self._rx_coarse_ddc_channel_names, "main_nco_frequency", False
        )

    @rx_main_nco_frequencies.setter
    def rx_main_nco_frequencies(self, value):
        self._set_iio_attr_int_vec(
            self._rx_coarse_ddc_channel_names, "main_nco_frequency", False, value,
        )

    @property
    def rx_main_nco_phases(self):
        """rx_main_nco_phases: Receive path coarse DDC NCO phases
        """
        return self._get_iio_attr_vec(
            self._rx_coarse_ddc_channel_names, "main_nco_phase", False
        )

    @rx_main_nco_phases.setter
    def rx_main_nco_phases(self, value):
        self._set_iio_attr_int_vec(
            self._rx_coarse_ddc_channel_names, "main_nco_phase", False, value,
        )

    @property
    def rx_test_mode(self):
        """rx_test_mode: NCO Test Mode """
        return self._get_iio_attr_str_single("voltage0_i", "test_mode", False)

    @rx_test_mode.setter
    def rx_test_mode(self, value):
        self._set_iio_attr_single(
            "voltage0_i", "test_mode", False, value,
        )

    @property
    def rx_nyquist_zone(self):
        """rx_nyquist_zone: ADC nyquist zone. Options are: odd, even """
        return self._get_iio_attr_str_single("voltage0_i", "nyquist_zone", False)

    @rx_nyquist_zone.setter
    def rx_nyquist_zone(self, value):
        self._set_iio_attr_single(
            "voltage0_i", "nyquist_zone", False, value,
        )

    @property
    def tx_channel_nco_frequencies(self):
        """tx_channel_nco_frequencies: Transmit path fine DUC NCO frequencies
        """
        return self._get_iio_attr_vec(
            self._tx_fine_duc_channel_names, "channel_nco_frequency", True
        )

    @tx_channel_nco_frequencies.setter
    def tx_channel_nco_frequencies(self, value):
        self._set_iio_attr_int_vec(
            self._tx_fine_duc_channel_names, "channel_nco_frequency", True, value
        )

    @property
    def tx_channel_nco_phases(self):
        """tx_channel_nco_phases: Transmit path fine DUC NCO phases
        """
        return self._get_iio_attr_vec(
            self._tx_fine_duc_channel_names, "channel_nco_phase", True
        )

    @tx_channel_nco_phases.setter
    def tx_channel_nco_phases(self, value):
        self._set_iio_attr_int_vec(
            self._tx_fine_duc_channel_names, "channel_nco_phase", True, value,
        )

    @property
    def tx_channel_nco_test_tone_en(self):
        """tx_channel_nco_test_tone_en: Transmit path fine DUC NCO test tone enable
        """
        return self._get_iio_attr_vec(
            self._tx_coarse_duc_channel_names, "channel_nco_test_tone_en", True
        )

    @tx_channel_nco_test_tone_en.setter
    def tx_channel_nco_test_tone_en(self, value):
        self._set_iio_attr_int_vec(
            self._tx_coarse_duc_channel_names, "channel_nco_test_tone_en", True, value,
        )

    @property
    def tx_channel_nco_test_tone_scales(self):
        """tx_channel_nco_test_tone_scales: Transmit path fine DUC NCO test tone scale
        """
        return self._get_iio_attr_vec(
            self._tx_coarse_duc_channel_names, "channel_nco_test_tone_scale", True
        )

    @tx_channel_nco_test_tone_scales.setter
    def tx_channel_nco_test_tone_scales(self, value):
        self._set_iio_attr_float_vec(
            self._tx_coarse_duc_channel_names,
            "channel_nco_test_tone_scale",
            True,
            value,
        )

    @property
    def tx_channel_nco_gain_scales(self):
        """tx_channel_nco_gain_scales Transmit path fine DUC NCO gain scale
        """
        return self._get_iio_attr_vec(
            self._tx_coarse_duc_channel_names, "channel_nco_gain_scale", True
        )

    @tx_channel_nco_gain_scales.setter
    def tx_channel_nco_gain_scales(self, value):
        self._set_iio_attr_float_vec(
            self._tx_coarse_duc_channel_names, "channel_nco_gain_scale", True, value,
        )

    @property
    def tx_main_nco_frequencies(self):
        """tx_main_nco_frequencies: Transmit path coarse DUC NCO frequencies
        """
        return self._get_iio_attr_vec(
            self._tx_coarse_duc_channel_names, "main_nco_frequency", True
        )

    @tx_main_nco_frequencies.setter
    def tx_main_nco_frequencies(self, value):
        self._set_iio_attr_int_vec(
            self._tx_coarse_duc_channel_names, "main_nco_frequency", True, value,
        )

    @property
    def tx_main_nco_phases(self):
        """tx_main_nco_phases: Transmit path coarse DUC NCO phases
        """
        return self._get_iio_attr_vec(
            self._tx_coarse_duc_channel_names, "main_nco_phase", True
        )

    @tx_main_nco_phases.setter
    def tx_main_nco_phases(self, value):
        self._set_iio_attr_int_vec(
            self._tx_coarse_duc_channel_names, "main_nco_phase", True, value,
        )

    @property
    def tx_main_nco_test_tone_en(self):
        """tx_main_nco_test_tone_en: Transmit path coarse DUC NCO test tone enable
        """
        return self._get_iio_attr_vec(
            self._tx_coarse_duc_channel_names, "main_nco_test_tone_en", True
        )

    @tx_main_nco_test_tone_en.setter
    def tx_main_nco_test_tone_en(self, value):
        self._set_iio_attr_int_vec(
            self._tx_coarse_duc_channel_names, "main_nco_test_tone_en", True, value,
        )

    @property
    def tx_main_nco_test_tone_scales(self):
        """tx_main_nco_test_tone_scales: Transmit path coarse DUC NCO test tone scale
        """
        return self._get_iio_attr_vec(
            self._tx_coarse_duc_channel_names, "main_nco_test_tone_scale", True
        )

    @tx_main_nco_test_tone_scales.setter
    def tx_main_nco_test_tone_scales(self, value):
        self._set_iio_attr_float_vec(
            self._tx_coarse_duc_channel_names, "main_nco_test_tone_scale", True, value,
        )

    @property
    def tx_main_ffh_frequency(self):
        """tx_main_ffh_frequency: Transmitter fast frequency hop frequency. This will set
            The NCO frequency of the NCO selected from the bank defined by tx_main_ffh_index
        """
        return self._get_iio_attr_single("voltage0_i", "main_ffh_frequency", True)

    @tx_main_ffh_frequency.setter
    def tx_main_ffh_frequency(self, value):
        if self.tx_main_ffh_index == 0:
            raise Exception(
                "To set a FFH NCO bank frequency, tx_main_ffh_index must > 0"
            )
        self._set_iio_attr_single(
            "voltage0_i", "main_ffh_frequency", True, value,
        )

    @property
    def tx_main_ffh_index(self):
        """tx_main_ffh_index: Transmitter fast frequency hop NCO bank index
        """
        return self._get_iio_attr_single("voltage0_i", "main_ffh_index", True)

    @tx_main_ffh_index.setter
    def tx_main_ffh_index(self, value):
        self._set_iio_attr_single(
            "voltage0_i", "main_ffh_index", True, value,
        )

    @property
    def tx_main_ffh_mode(self):
        """tx_main_ffh_mode: Set hop transition mode of NCOs Options are:
            phase_continuous, phase_incontinuous, and phase_coherent
        """
        return self._get_iio_attr_str_single("voltage0_i", "main_ffh_mode", True)

    @tx_main_ffh_mode.setter
    def tx_main_ffh_mode(self, value):
        self._set_iio_attr_single(
            "voltage0_i", "main_ffh_mode", True, value,
        )

    @property
    def loopback_mode(self):
        """loopback_mode: Enable loopback mode RX->TX
        """
        return self._get_iio_dev_attr_single("loopback_mode")

    @loopback_mode.setter
    def loopback_mode(self, value):
        self._set_iio_dev_attr_single(
            "loopback_mode", value,
        )

    @property
    def rx_sample_rate(self):
        """rx_sampling_frequency: Sample rate after decimation"""
        return self._get_iio_attr_single("voltage0_i", "sampling_frequency", False)

    @property
    def adc_frequency(self):
        """adc_frequency: ADC frequency in Hz"""
        return self._get_iio_attr_single("voltage0_i", "adc_frequency", False)

    @property
    def tx_sample_rate(self):
        """tx_sampling_frequency: Sample rate before interpolation"""
        return self._get_iio_attr_single("voltage0_i", "sampling_frequency", True)

    @property
    def dac_frequency(self):
        """dac_frequency: DAC frequency in Hz"""
        return self._get_iio_attr_single("voltage0_i", "dac_frequency", True)



import adi
import pytest

hardware = ["ad9371", "ad9144"]
classname = "adi.sshfs.sshfs"


def open_sshfs(classname, iio_uri, username, password):
    _password = f'"{password}"' if password is not None else None
    return eval(f'{classname}("{iio_uri}", "{username}", {_password})')


@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
def test_sshfs_isfile(iio_uri, classname, username, password):
    sshfs = open_sshfs(classname, iio_uri, username, password)
    assert sshfs.isfile("/etc/os-release")
    assert sshfs.isfile("/etc/") == False


@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.iio_hardware(hardware)
def test_sshfs_listdir(iio_uri, classname, username, password):
    sshfs = open_sshfs(classname, iio_uri, username, password)
    assert isinstance(sshfs.listdir("/etc/"), list)


@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.iio_hardware(hardware)
def test_sshfs_gettext(iio_uri, classname, username, password):
    sshfs = open_sshfs(classname, iio_uri, username, password)
    assert sshfs.gettext("/proc/version").startswith("Linux version")



import pytest

hardware = "ad7606b"
classname = "adi.ad7606"


#########################################
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize(
    "attr, max_pow, tol", [("oversampling_ratio", 6, 0),],
)
def test_ad7606_attr(
    test_attribute_single_value_pow2, iio_uri, classname, attr, max_pow, tol
):
    test_attribute_single_value_pow2(iio_uri, classname, attr, max_pow, tol)


#########################################
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("channel", range(2))
def test_ad7606_rx_data(test_dma_rx, iio_uri, classname, channel):
    test_dma_rx(iio_uri, classname, channel)



import re

import setuptools


# From: https://github.com/smartcar/python-sdk/blob/master/setup.py
def _get_version():
    """Extract version from package."""
    with open("adi/__init__.py") as reader:
        match = re.search(
            r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]', reader.read(), re.MULTILINE
        )
        if match:
            return match.group(1)
        else:
            raise RuntimeError("Unable to extract version.")


with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="pyadi-iio",
    version=_get_version(),
    author="Travis Collins",
    author_email="travis.collins@analog.com",
    description="Interfaces to stream data from ADI hardware",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/analogdevicesinc/pyadi-iio",
    packages=setuptools.find_packages(exclude=["test*"]),
    python_requires=">=3.6",
    install_requires=["numpy", "pylibiio<=0.21", "paramiko"],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
    ],
)



from os import sendfile


def get_test_map():
    # Keys are substrings of test function names (and usually filenames)
    # Any board-fmc name will run a specific tests with key as name

    # NO TESTS YET
    # zynq-zc702-adv7511
    # zynq-zc706-adv7511-fmcadc4
    # zynq-zc706-adv7511-fmcomms11
    # zynq-zc706-adv7511-ad9625-fmcadc2
    #
    #
    # zynq-zed-adv7511-ad9467-fmc-250ebz
    # zynq-zc706-adv7511-ad9625-fmcadc3
    #
    # zynq-zed-adv7511-cn0363
    # zynq-zc706-adv7511-ad6676-fmc
    #
    # zynq-zc706-adv7511-ad9265-fmc-125ebz
    # zynq-zc706-adv7511-ad9434-fmc-500ebz
    # zynq-zed-adv7511
    # zynq-zc706-adv7511-ad9739a-fmc
    # zynq-zed-imageon
    # zynq-zc706-adv7511-fmcdaq3-revC
    # zynq-zc706-adv7511-fmcjesdadc1
    # zynq-zed-adv7511-fmcmotcon2
    # zynq-zc706-adv7511

    test_map = {}
    test_map["daq3"] = ["zynqmp-zcu102-rev10-fmcdaq3"]
    test_map["ad9152"] = ["zynqmp-zcu102-rev10-fmcdaq3"]
    test_map["daq2"] = ["zynq-zc706-adv7511-fmcdaq2", "zynqmp-zcu102-rev10-fmcdaq2"]
    test_map["ad9144"] = ["zynq-zc706-adv7511-fmcdaq2", "zynqmp-zcu102-rev10-fmcdaq2"]
    test_map["ad9680"] = ["zynq-zc706-adv7511-fmcdaq2", "zynqmp-zcu102-rev10-fmcdaq2"]
    test_map["adrv9371"] = [
        "zynqmp-zcu102-rev10-adrv9371",
        "zynq-zc706-adv7511-adrv9371",
        "socfpga_arria10_socdk_adrv9371",
    ]
    test_map["adrv9009"] = [
        "socfpga_arria10_socdk_adrv9009",
        "zynqmp-zcu102-rev10-adrv9008-1",
        "zynqmp-zcu102-rev10-adrv9008-2",
        "zynqmp-zcu102-rev10-adrv9009",
    ]
    test_map["fmcomms5"] = [
        "zynqmp-zcu102-rev10-ad9361-fmcomms5",
        "zynq-zc702-adv7511-ad9361-fmcomms5",
        "zynq-zc706-adv7511-ad9361-fmcomms5",
        "zynq-zc706-adv7511-ad9361-fmcomms5-ext-lo-adf5355",
    ]
    test_map["pluto"] = ["pluto"]
    test_map["ad9361"] = [
        "socfpga_cyclone5_sockit_arradio",
        "zynq-zed-adv7511-ad9361-fmcomms2-3",
        "zynq-zc702-adv7511-ad9361-fmcomms2-3",
        "zynq-zc706-adv7511-ad9361-fmcomms2-3",
        "zynqmp-zcu102-rev10-ad9361-fmcomms2-3",
        "zynq-adrv9361-z7035-fmc",
        "zynq-adrv9361-z7035-box",
        "zynq-adrv9361-z7035-bob",
        "zynq-adrv9361-z7035-bob-cmos",
    ] + test_map["fmcomms5"]
    test_map["ad9364"] = [
        "socfpga_cyclone5_sockit_arradio",
        "zynq-zc702-adv7511-ad9364-fmcomms4",
        "zynq-zc706-adv7511-ad9364-fmcomms4",
        "zynqmp-zcu102-rev10-ad9364-fmcomms4",
        "zynq-adrv9364-z7020-box",
        "zynq-adrv9364-z7020-bob",
        "zynq-adrv9364-z7020-bob-cmos",
        "zynq-zed-adv7511-ad9364-fmcomms4",
    ]
    test_map["adrv9002"] = [
        "zynq-zc706-adv7511-adrv9002",
        "zynq-zc706-adv7511-adrv9002-rx2tx2",
        "zynq-zed-adv7511-adrv9002",
        "zynq-zed-adv7511-adrv9002-rx2tx2",
        "zynqmp-zcu102-rev10-adrv9002",
        "zynqmp-zcu102-rev10-adrv9002-rx2tx2",
    ]

    return test_map



import pytest

hardware = "adpd410x"
classname = "adi.adpd410x"

#########################################
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("channel", range(8))
def test_adpd410x_rx_data(test_dma_rx, iio_uri, classname, channel):
    test_dma_rx(iio_uri, classname, channel)



import pytest

hardware = "adrv9009-dual"
classname = "adi.adrv9009_zu11eg"


#########################################
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize(
    "attr, start, stop, step, tol",
    [
        ("tx_hardwaregain_chan0", -41.95, 0.0, 0.05, 0.05),
        ("tx_hardwaregain_chan1", -41.95, 0.0, 0.05, 0.05),
        ("tx_hardwaregain_chan0_chip_b", -41.95, 0.0, 0.05, 0.05),
        ("tx_hardwaregain_chan1_chip_b", -41.95, 0.0, 0.05, 0.05),
        ("trx_lo", 70000000, 6000000000, 1000, 0),
        ("trx_lo_chip_b", 70000000, 6000000000, 1000, 0),
    ],
)
def test_adrv9009_zu11eg_attr(
    test_attribute_single_value, iio_uri, classname, attr, start, stop, step, tol
):
    test_attribute_single_value(iio_uri, classname, attr, start, stop, step, tol)


#########################################
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("channel", range(4))
def test_adrv9009_zu11eg_rx_data(test_dma_rx, iio_uri, classname, channel):
    test_dma_rx(iio_uri, classname, channel)


#########################################
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("channel", [0, 1, 2, 3])
@pytest.mark.parametrize(
    "param_set",
    [
        dict(
            trx_lo=1000000000,
            trx_lo_chip_b=5000000000,
            gain_control_mode_chan0="slow_attack",
            gain_control_mode_chan1="slow_attack",
            gain_control_mode_chan0_chip_b="slow_attack",
            gain_control_mode_chan1_chip_b="slow_attack",
            tx_hardwaregain_chan0=-10,
            tx_hardwaregain_chan1=-10,
            tx_hardwaregain_chan0_chip_b=-10,
            tx_hardwaregain_chan1_chip_b=-10,
            calibrate_rx_qec_en=1,
            calibrate_tx_qec_en=1,
            calibrate=1,
            calibrate_rx_qec_en_chip_b=1,
            calibrate_tx_qec_en_chip_b=1,
            calibrate_chip_b=1,
        ),
        dict(
            trx_lo=3000000000,
            trx_lo_chip_b=4000000000,
            gain_control_mode_chan0="slow_attack",
            gain_control_mode_chan1="slow_attack",
            gain_control_mode_chan0_chip_b="slow_attack",
            gain_control_mode_chan1_chip_b="slow_attack",
            tx_hardwaregain_chan0=-10,
            tx_hardwaregain_chan1=-10,
            tx_hardwaregain_chan0_chip_b=-10,
            tx_hardwaregain_chan1_chip_b=-10,
            calibrate_rx_qec_en=1,
            calibrate_tx_qec_en=1,
            calibrate=1,
            calibrate_rx_qec_en_chip_b=1,
            calibrate_tx_qec_en_chip_b=1,
            calibrate_chip_b=1,
        ),
        dict(
            trx_lo=5000000000,
            trx_lo_chip_b=1000000000,
            gain_control_mode_chan0="slow_attack",
            gain_control_mode_chan1="slow_attack",
            gain_control_mode_chan0_chip_b="slow_attack",
            gain_control_mode_chan1_chip_b="slow_attack",
            tx_hardwaregain_chan0=-10,
            tx_hardwaregain_chan1=-10,
            tx_hardwaregain_chan0_chip_b=-10,
            tx_hardwaregain_chan1_chip_b=-10,
            calibrate_rx_qec_en=1,
            calibrate_tx_qec_en=1,
            calibrate=1,
            calibrate_rx_qec_en_chip_b=1,
            calibrate_tx_qec_en_chip_b=1,
            calibrate_chip_b=1,
        ),
    ],
)
@pytest.mark.parametrize("sfdr_min", [45])
def test_adrv9009_zu11eg_sfdr(
    test_sfdr, iio_uri, classname, channel, param_set, sfdr_min
):
    test_sfdr(iio_uri, classname, channel, param_set, sfdr_min)


#########################################
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("channel", [0, 1, 2, 3])
@pytest.mark.parametrize(
    "param_set",
    [
        dict(
            trx_lo=1000000000,
            trx_lo_chip_b=1000000000,
            gain_control_mode_chan0="slow_attack",
            gain_control_mode_chan1="slow_attack",
            gain_control_mode_chan0_chip_b="slow_attack",
            gain_control_mode_chan1_chip_b="slow_attack",
            tx_hardwaregain_chan0=-10,
            tx_hardwaregain_chan1=-10,
            tx_hardwaregain_chan0_chip_b=-10,
            tx_hardwaregain_chan1_chip_b=-10,
            calibrate_rx_qec_en=1,
            calibrate_tx_qec_en=1,
            calibrate=1,
            calibrate_rx_qec_en_chip_b=1,
            calibrate_tx_qec_en_chip_b=1,
            calibrate_chip_b=1,
        ),
        dict(
            trx_lo=3000000000,
            trx_lo_chip_b=1000000000,
            gain_control_mode_chan0="slow_attack",
            gain_control_mode_chan1="slow_attack",
            gain_control_mode_chan0_chip_b="slow_attack",
            gain_control_mode_chan1_chip_b="slow_attack",
            tx_hardwaregain_chan0=-10,
            tx_hardwaregain_chan1=-10,
            tx_hardwaregain_chan0_chip_b=-10,
            tx_hardwaregain_chan1_chip_b=-10,
            calibrate_rx_qec_en=1,
            calibrate_tx_qec_en=1,
            calibrate=1,
            calibrate_rx_qec_en_chip_b=1,
            calibrate_tx_qec_en_chip_b=1,
            calibrate_chip_b=1,
        ),
        dict(
            trx_lo=5000000000,
            trx_lo_chip_b=1000000000,
            gain_control_mode_chan0="slow_attack",
            gain_control_mode_chan1="slow_attack",
            gain_control_mode_chan0_chip_b="slow_attack",
            gain_control_mode_chan1_chip_b="slow_attack",
            tx_hardwaregain_chan0=-10,
            tx_hardwaregain_chan1=-10,
            tx_hardwaregain_chan0_chip_b=-10,
            tx_hardwaregain_chan1_chip_b=-10,
            calibrate_rx_qec_en=1,
            calibrate_tx_qec_en=1,
            calibrate=1,
            calibrate_rx_qec_en_chip_b=1,
            calibrate_tx_qec_en_chip_b=1,
            calibrate_chip_b=1,
        ),
    ],
)
@pytest.mark.parametrize("dds_scale, min_rssi, max_rssi", [(0, 35, 60), (0.9, 0, 22)])
def test_adrv9009_zu11eg_dds_gain_check_agc(
    test_gain_check,
    iio_uri,
    classname,
    channel,
    param_set,
    dds_scale,
    min_rssi,
    max_rssi,
):
    test_gain_check(
        iio_uri, classname, channel, param_set, dds_scale, min_rssi, max_rssi
    )


#########################################
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("channel", [0, 1, 2, 3])
@pytest.mark.parametrize(
    "param_set, dds_scale, min_rssi, max_rssi",
    [
        (
            dict(
                trx_lo=1000000000,
                trx_lo_chip_b=1000000000,
                gain_control_mode_chan0="manual",
                gain_control_mode_chan1="manual",
                gain_control_mode_chan0_chip_b="manual",
                gain_control_mode_chan1_chip_b="manual",
                rx_hardwaregain_chan0=0,
                rx_hardwaregain_chan1=0,
                rx_hardwaregain_chan0_chip_b=0,
                rx_hardwaregain_chan1_chip_b=0,
                tx_hardwaregain_chan0=-10,
                tx_hardwaregain_chan1=-10,
                tx_hardwaregain_chan0_chip_b=-10,
                tx_hardwaregain_chan1_chip_b=-10,
                calibrate_tx_qec_en=1,
                calibrate_tx_qec_en_chip_b=1,
                calibrate=1,
                calibrate_chip_b=1,
            ),
            0.5,
            20,
            60,
        ),
        (
            dict(
                trx_lo=1000000000,
                gain_control_mode_chan0="manual",
                gain_control_mode_chan1="manual",
                gain_control_mode_chan0_chip_b="manual",
                gain_control_mode_chan1_chip_b="manual",
                rx_hardwaregain_chan0=30,
                rx_hardwaregain_chan1=30,
                rx_hardwaregain_chan0_chip_b=30,
                rx_hardwaregain_chan1_chip_b=30,
                tx_hardwaregain_chan0=-10,
                tx_hardwaregain_chan1=-10,
                tx_hardwaregain_chan0_chip_b=-10,
                tx_hardwaregain_chan1_chip_b=-10,
                calibrate_tx_qec_en=1,
                calibrate_tx_qec_en_chip_b=1,
                calibrate=1,
                calibrate_chip_b=1,
            ),
            0.5,
            0,
            15,
        ),
    ],
)
def test_adrv9009_zu11eg_dds_gain_check_vary_power(
    test_gain_check,
    iio_uri,
    classname,
    channel,
    param_set,
    dds_scale,
    min_rssi,
    max_rssi,
):
    test_gain_check(classname, channel, param_set, dds_scale, min_rssi, max_rssi)


#########################################
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("channel", [0, 1, 2, 3])
@pytest.mark.parametrize(
    "param_set",
    [
        dict(
            trx_lo=1000000000,
            trx_lo_chip_b=5000000000,
            gain_control_mode_chan0="slow_attack",
            gain_control_mode_chan1="slow_attack",
            gain_control_mode_chan0_chip_b="slow_attack",
            gain_control_mode_chan1_chip_b="slow_attack",
            tx_hardwaregain_chan0=-20,
            tx_hardwaregain_chan1=-20,
            tx_hardwaregain_chan0_chip_b=-20,
            tx_hardwaregain_chan1_chip_b=-20,
            calibrate_tx_qec_en=1,
            calibrate_tx_qec_en_chip_b=1,
            calibrate=1,
            calibrate_chip_b=1,
        ),
        dict(
            trx_lo=3000000000,
            trx_lo_chip_b=4000000000,
            gain_control_mode_chan0="slow_attack",
            gain_control_mode_chan1="slow_attack",
            gain_control_mode_chan0_chip_b="slow_attack",
            gain_control_mode_chan1_chip_b="slow_attack",
            tx_hardwaregain_chan0=-20,
            tx_hardwaregain_chan1=-20,
            tx_hardwaregain_chan0_chip_b=-20,
            tx_hardwaregain_chan1_chip_b=-20,
            calibrate_tx_qec_en=1,
            calibrate_tx_qec_en_chip_b=1,
            calibrate=1,
            calibrate_chip_b=1,
        ),
        dict(
            trx_lo=5000000000,
            trx_lo_chip_b=1000000000,
            gain_control_mode_chan0="slow_attack",
            gain_control_mode_chan1="slow_attack",
            gain_control_mode_chan0_chip_b="slow_attack",
            gain_control_mode_chan1_chip_b="slow_attack",
            tx_hardwaregain_chan0=-20,
            tx_hardwaregain_chan1=-20,
            tx_hardwaregain_chan0_chip_b=-20,
            tx_hardwaregain_chan1_chip_b=-20,
            calibrate_tx_qec_en=1,
            calibrate_tx_qec_en_chip_b=1,
            calibrate=1,
            calibrate_chip_b=1,
        ),
    ],
)
def test_adrv9009_zu11eg_iq_loopback(
    test_iq_loopback, iio_uri, classname, channel, param_set
):
    test_iq_loopback(iio_uri, classname, channel, param_set)



from test.generics import compare_states, get_states, iio_buffer_check

import iio

import pytest

hardware = ["pluto", "adrv9361", "fmcomms2"]
classname = ""


#########################################
# fmt: off
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize(
    "attrtype, dev_name, chan_name, inout, attr, start, stop, step, tol, repeats",
    [
        ("channel","ad9361-phy","voltage0",False,"hardwaregain",-89.75,0.0,0.25,0,100,),
        ("channel","ad9361-phy","TX_LO",True,"frequency",47000000,6000000000,1,4,100,),
        ("channel","ad9361-phy","RX_LO",True,"frequency",70000000,6000000000,1,4,100,),
    ],
)
def test_iio_attr(
    test_iio_attribute_single_value,iio_uri,attrtype,dev_name,chan_name,inout,
    attr,start,stop,step,tol,repeats,
):
    test_iio_attribute_single_value(
        iio_uri,attrtype,dev_name,chan_name,inout,
        attr,start,stop,step,tol,repeats,
    )
# fmt: on

#########################################
@pytest.mark.iio_hardware(hardware)
def test_attribute_changes(context_desc):

    ctx = None
    for ctx_desc in context_desc:
        if ctx_desc["hw"] in hardware:
            ctx = iio.Context(ctx_desc["uri"])
    if not ctx:
        pytest.skip("No valid hardware found")

    drivers_to_ignore = "xadc"

    # Set initial state
    ctx.find_device("ad9361-phy").find_channel("RX_LO").attrs[
        "frequency"
    ].value = "1000000000"
    ctx.find_device("ad9361-phy").find_channel("TX_LO").attrs[
        "frequency"
    ].value = "1000000000"

    # Collect state of all attributes
    state1 = get_states(ctx, drivers_to_ignore)

    # Change LOs
    ctx.find_device("ad9361-phy").find_channel("RX_LO").attrs[
        "frequency"
    ].value = "2000000000"
    ctx.find_device("ad9361-phy").find_channel("TX_LO").attrs[
        "frequency"
    ].value = "2000000000"

    # Collect state of all attributes after change
    state2 = get_states(ctx, drivers_to_ignore)

    # Set up comparison
    expected_to_change = [
        "ad9361-phy_out_altvoltage0_frequency",
        "ad9361-phy_out_altvoltage1_frequency",
    ]
    allowed_to_change = [
        "ad7291_in_temp0_mean_raw",
        "ad7291_in_temp0_raw",
        "ad9361-phy_in_temp0_input",
        "ad9361-phy_in_voltage0_hardwaregain",
        "ad9361-phy_in_voltage1_hardwaregain",
        "ad9361-phy_in_voltage2_raw",
        "ad9361-phy_in_voltage0_rssi",
        "ad9361-phy_in_voltage1_rssi",
        "ad9361-phy_in_voltage0_hardwaregain_available",
        "ad9361-phy_in_voltage1_hardwaregain_available",
    ]
    for k in range(6):
        allowed_to_change.append("ad7291_in_voltage{}_raw".format(k))

    compare_states(state1, state2, expected_to_change, allowed_to_change)


#########################################


@pytest.mark.parametrize("percent_fail", [(0.1)])
@pytest.mark.iio_hardware(hardware)
def test_iio_buffer_check(single_ctx_desc, percent_fail):
    iio_buffer_check(
        "ad9361-phy", "cf-ad9361-lpc", single_ctx_desc["uri"], percent_fail
    )



import iio

import adi
import numpy as np
import pytest

hardware = ["packrf", "adrv9361", "fmcomms3", "ad9361"]
classname = "adi.ad9361"

##################################
@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize(
    "voltage_raw, low, high",
    [
        ("in_temp0", 80, 160),
        ("in_voltage0", 3045, 3629),
        ("in_voltage1", 3045, 3629),
        ("in_voltage2", 3045, 3629),
        ("in_voltage3", 1231, 1393),
        ("in_voltage4", 1661, 2749),
        ("in_voltage5", 1231, 1393),
    ],
)
def test_ad7291(context_desc, voltage_raw, low, high):
    ctx = None
    for ctx_desc in context_desc:
        if ctx_desc["hw"] in hardware:
            ctx = iio.Context(ctx_desc["uri"])
    if not ctx:
        pytest.skip("No valid hardware found")

    ad7291 = ctx.find_device("ad7291")

    for channel in ad7291.channels:
        c_name = "out" if channel.output else "in"
        c_name += "_" + str(channel.id)
        if c_name == voltage_raw:
            for attr in channel.attrs:
                if attr == "raw":
                    try:
                        print(channel.attrs[attr].value)
                        assert low <= int(channel.attrs[attr].value) <= high
                    except OSError:
                        continue


@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize(
    "attr, start, stop, step, tol, repeats",
    [
        ("tx_hardwaregain_chan0", -30.0, -7.0, 0.25, 0, 100),
        ("tx_hardwaregain_chan1", -30.0, -7.0, 0.25, 0, 100),
        ("rx_lo", 2300000000, 2500000000, 1, 8, 100),
        ("tx_lo", 2300000000, 2500000000, 1, 8, 100),
        ("sample_rate", 30700000, 30740000, 1, 4, 20),
        ("rx_rf_bandwidth", 16000000, 19000000, 1, 4, 10),
        ("tx_rf_bandwidth", 16000000, 19000000, 1, 4, 10),
    ],
)
def test_ad9361_attr(
    test_attribute_single_value,
    iio_uri,
    classname,
    attr,
    start,
    stop,
    step,
    tol,
    repeats,
):
    test_attribute_single_value(
        iio_uri, classname, attr, start, stop, step, tol, repeats
    )


@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("channel", [0, 1])
@pytest.mark.parametrize(
    "dds_scale, min_rssi, max_rssi, param_set",
    [
        (
            0.0,
            75,
            150,
            dict(
                sample_rate=30720000,
                tx_lo=2300000000,
                rx_lo=2400000000,
                gain_control_mode_chan0="slow_attack",
                gain_control_mode_chan1="slow_attack",
                rx_rf_bandwidth=18000000,
                tx_rf_bandwidth=18000000,
            ),
        ),
        (
            0.4,
            10,
            50,
            dict(
                gain_control_mode_chan0="slow_attack",
                gain_control_mode_chan1="slow_attack",
                rx_lo=2400000000,
                tx_lo=2400000000,
                tx_hardwaregain_chan0=-10,
                tx_hardwaregain_chan1=-10,
                sample_rate=30720000,
                rx_rf_bandwidth=18000000,
                tx_rf_bandwidth=18000000,
            ),
        ),
    ],
)
def test_rssi(
    test_gain_check,
    iio_uri,
    classname,
    channel,
    param_set,
    dds_scale,
    min_rssi,
    max_rssi,
):
    test_gain_check(
        iio_uri, classname, channel, param_set, dds_scale, min_rssi, max_rssi
    )


@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("channel", [0, 1])
@pytest.mark.parametrize(
    "dds_scale, frequency, hardwaregain_low, hardwaregain_high",
    [(0.0, 999859, 50, 80), (0.4, 999859, 0.0, 28)],
)
def test_hardware_gain(
    test_hardwaregain,
    iio_uri,
    classname,
    channel,
    dds_scale,
    frequency,
    hardwaregain_low,
    hardwaregain_high,
):
    test_hardwaregain(
        iio_uri,
        classname,
        channel,
        dds_scale,
        frequency,
        hardwaregain_low,
        hardwaregain_high,
    )


@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("channel", [0, 1])
def test_ad9361_loopback(test_dma_loopback, iio_uri, classname, channel):
    test_dma_loopback(iio_uri, classname, channel)


@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("channel", [0, 1])
@pytest.mark.parametrize(
    "param_set",
    [
        dict(
            sample_rate=30720000,
            tx_lo=2400000000,
            rx_lo=2400000000,
            gain_control_mode_chan0="slow_attack",
            gain_control_mode_chan1="slow_attack",
            tx_hardwaregain_chan0=-10,
            tx_hardwaregain_chan1=-10,
            rx_rf_bandwidth=18000000,
            tx_rf_bandwidth=18000000,
        ),
    ],
)
def test_ad9361_iq_loopback(test_iq_loopback, iio_uri, classname, channel, param_set):
    test_iq_loopback(iio_uri, classname, channel, param_set)


@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
def test_dcxo(test_dcxo_calibration, classname, iio_uri):
    test_dcxo_calibration(classname, iio_uri)


@pytest.mark.iio_hardware(hardware)
@pytest.mark.parametrize("classname", [(classname)])
@pytest.mark.parametrize("channel", [0, 1])
@pytest.mark.parametrize(
    "param_set",
    [
        dict(
            tx_lo=2400000000,
            rx_lo=2400000000,
            tx_hardwaregain_chan0=-10,
            tx_hardwaregain_chan1=-10,
            sample_rate=30720000,
        )
    ],
)
@pytest.mark.parametrize(
    "low, high",
    [([-20.0, -100.0, -110.0, -110.0, -110.0], [-10.0, -60.0, -75.0, -75.0, -80.0])],
)
def test_harmonic_values(
    test_harmonics, classname, iio_uri, channel, param_set, low, high, plot=False
):
    test_harmonics(classname, iio_uri, channel, param_set, low, high, plot)
